__author_email__ = 'maintainers@eveinsight.net'
__description__ = 'ESI celery task queues for distributed API access using RabbitMQ and Redis.'
__license__ = 'GNU General Public License v3.0'
__title__ = 'EVECelery'
__url__ = 'https://github.com/NatEVETools/EVECelery'
__version__ = '0.14'
